/*****************************************************************

  GlideControl.c

  Copyright (c) 1998 by Mike Oliphant - oliphant@ling.ed.ac.uk

    http://www.ling.ed.ac.uk/~oliphant/gc

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
#include <glide.h>

void OKDialog(char *message);
void BoolDialog(char *question,char *yes,GtkSignalFunc yesfunc,
				char *no,GtkSignalFunc nofunc);
void ShutDown(GtkWidget *widget, GdkEvent *event, gpointer data);
void ShutDownFerSher(GtkWidget *widget, GdkEvent *event, gpointer data);
GtkWidget *MakeNewPage(char *name);
void MakeInfoPage(void);
void MakeCheckButton(GtkWidget *box,char *name,int num);
void ChangeIntVal(GtkWidget *widget,gpointer data);
void MakeNumEntry(GtkWidget *box,char *name,int num,int len);
void ChangeBoolVal(GtkWidget *widget,gpointer data);
void MakeGenPage(void);
void MakeSpeedPage(void);
void MakeColPage(void);
void MakeAboutPage(void);
void GammaSlide(GtkWidget *widget,gpointer data);
void MakeGammaWidget(GtkWidget *box,char *name,int num);
void ReadBoolSet(char *name,int *val);
void ReadIntSet(char *name,int *val);
void ReadFloatSet(char *name,float *val);
void ReadSettings(void);
void SaveBoolSet(char *name,int val,FILE *fp);
void SaveIntSet(char *name,int val,FILE *fp);
void SaveFloatSet(char *name,float val,FILE *fp);
void SaveSettings(FILE *fp);
void DoSave(GtkWidget *widget,gpointer data);

static GrHwConfiguration hwconfig;
static char version[80];

int voodoo2=0;

int nosplash=1;
int shameplug=0;
int vgapass=0;
int noshutdown=0;

int swapint=1;
int fastmem=0;
int fastpcird=0;
int pcird;
int grxclk=50;
int refresh=60;
int vsync=1;


float maingamma=1.0;
float rgamma=1.0;
float ggamma=1.0;
float bgamma=1.0;

GtkWidget *window;
GtkWidget *notebook;
GtkWidget *infopage;
GtkWidget *genpage;
GtkWidget *colpage;
GtkWidget *speedpage;
GtkWidget *aboutpage;

/* Boolean variables */

int *checkvals[]={&nosplash,&shameplug,&fastmem,&fastpcird,&vsync,
				  &vgapass,&noshutdown};
GtkWidget *checkbuttons[7];

/* Integer variables */

int *intvals[]={&swapint,&grxclk,&refresh};
GtkWidget *intentries[3];

/* Gamma variables */

float *gammavals[4]={&maingamma,&rgamma,&ggamma,&bgamma};
GtkWidget *gammaentries[4];
GtkObject *gammaadjs[4];

int changed=0;

void OKDialog(char *message)
{
  BoolDialog(message,"OK",NULL,NULL,NULL);
}

void BoolDialog(char *question,char *yes,GtkSignalFunc yesfunc,
				char *no,GtkSignalFunc nofunc)
{
  GtkWidget *dialog;
  GtkWidget *label;
  GtkWidget *yesbutton;
  GtkWidget *nobutton;

  dialog=gtk_dialog_new();

  gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),5);

  label=gtk_label_new(question);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),label,TRUE,TRUE,0);
  gtk_widget_show(label);

  yesbutton=gtk_button_new_with_label(yes);
  if(yesfunc)
	gtk_signal_connect(GTK_OBJECT(yesbutton),"clicked",
					   yesfunc,NULL);
  gtk_signal_connect_object(GTK_OBJECT(yesbutton),"clicked",
							GTK_SIGNAL_FUNC(gtk_widget_destroy),
							GTK_OBJECT(dialog));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),yesbutton,
					 TRUE,TRUE,0);
  gtk_widget_show(yesbutton);

  if(no) {
	nobutton=gtk_button_new_with_label(no);
	if(nofunc)
	  gtk_signal_connect(GTK_OBJECT(nobutton),"clicked",
						 nofunc,NULL);
	gtk_signal_connect_object(GTK_OBJECT(nobutton),"clicked",
							  GTK_SIGNAL_FUNC(gtk_widget_destroy),
							  GTK_OBJECT(dialog));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),nobutton,
					   TRUE,TRUE,0);
	gtk_widget_show(nobutton);
  }

  gtk_widget_show(dialog);

  gtk_grab_add(dialog);
}

void ShutDown(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  if(changed)
	BoolDialog("Settings have changed. Quit anyway?",
			   "Yes",GTK_SIGNAL_FUNC(ShutDownFerSher),"No",NULL);
  else
	ShutDownFerSher(widget,event,data);
}

void ShutDownFerSher(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_main_quit();
}
  
GtkWidget *MakeNewPage(char *name)
{
  GtkWidget *page;
  GtkWidget *label;

  page=gtk_frame_new(NULL);
  gtk_container_border_width(GTK_CONTAINER(page),5);
  gtk_widget_show(page);

  label=gtk_label_new(name);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),page,label);

  return page;
}

void MakeInfoPage(void)
{
  char buf[80];
  GtkWidget *label;
  GtkWidget *vbox;

  infopage=MakeNewPage("Info");

  vbox=gtk_vbox_new(TRUE,0);
  gtk_container_border_width(GTK_CONTAINER(vbox),5);

  sprintf(buf,"Glide version %s",version);
  label=gtk_label_new(buf);
  gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
  gtk_widget_show(label);

  switch(hwconfig.SSTs[0].type) {
  case GR_SSTTYPE_VOODOO:
	strcpy(buf,"Board type is Voodoo1");
	break;
  case GR_SSTTYPE_SST96:
	strcpy(buf,"Board type is SST96");
	break;
  case GR_SSTTYPE_AT3D:
	strcpy(buf,"Board type is AT3D");
	break;
  case GR_SSTTYPE_Voodoo2:
	voodoo2=1;
	sprintf(buf,"Board type is Voodoo2");
	break;
  }
  label=gtk_label_new(buf);
  gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
  gtk_widget_show(label);

  sprintf(buf,"%d TMU%s detected",hwconfig.num_sst,
		  (hwconfig.num_sst>1)?"s":"\0");
  label=gtk_label_new(buf);
  gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
  gtk_widget_show(label);

  if(hwconfig.SSTs[0].type==GR_SSTTYPE_VOODOO ||
	 hwconfig.SSTs[0].type==GR_SSTTYPE_Voodoo2) {
	sprintf(buf,"%dMB video ram",hwconfig.SSTs[0].sstBoard.VoodooConfig.fbRam);
	label=gtk_label_new(buf);
	gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
	gtk_widget_show(label);

	sprintf(buf,"Pixelfx rev %d",
			hwconfig.SSTs[0].sstBoard.VoodooConfig.fbiRev);
	label=gtk_label_new(buf);
	gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
	gtk_widget_show(label);

	sprintf(buf,"%d texelfx chip%s",
			hwconfig.SSTs[0].sstBoard.VoodooConfig.nTexelfx,
			(hwconfig.SSTs[0].sstBoard.VoodooConfig.nTexelfx>1)?"s":"\0");
	label=gtk_label_new(buf);
	gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
	gtk_widget_show(label);

	if(hwconfig.SSTs[0].sstBoard.VoodooConfig.sliDetect==FXTRUE) {
	  strcpy(buf,"SLI mode active");
	  label=gtk_label_new(buf);
	  gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);
	  gtk_widget_show(label);
	}
  }

  gtk_container_add(GTK_CONTAINER(infopage),vbox);
  gtk_widget_show(vbox);
}

void MakeSpeedPage(void)
{
  GtkWidget *vbox;

  speedpage=MakeNewPage("Speed");

  vbox=gtk_vbox_new(TRUE,0);

  gtk_container_border_width(GTK_CONTAINER(vbox),5);

  MakeCheckButton(vbox,"Enable fast memory",2);
  MakeCheckButton(vbox,"Enable fast PCI read",3);
  MakeCheckButton(vbox,"Sync to vertical refresh",4);
  MakeNumEntry(vbox,"Graphics clock speed",1,3);
  MakeNumEntry(vbox,"Swap interval",0,3);
  MakeNumEntry(vbox,"Refresh rate",2,3);

  gtk_container_add(GTK_CONTAINER(speedpage),vbox);
  gtk_widget_show(vbox);
}

void ChangeIntVal(GtkWidget *widget,gpointer data)
{
  char buf[80];
  int num;

  num=(int)data;

  *(intvals[num])=atoi(gtk_entry_get_text(GTK_ENTRY(widget)));

  if(num==1) {
	if((!voodoo2&&*(intvals[num])>57) || (voodoo2&&*(intvals[num])>95)) {
	  sprintf(buf,
			  "Warning!\nClock values above %d may damage your video card",
			  (voodoo2)?95:57);
	  
	  OKDialog(buf);
	}
  }

  changed=1;
}

void MakeNumEntry(GtkWidget *box,char *name,int num,int len)
{
  char buf[80];
  GtkWidget *label;
  GtkWidget *hbox;

  hbox=gtk_hbox_new(TRUE,5);

  label=gtk_label_new(name);
  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
  gtk_widget_show(label);

  intentries[num]=gtk_entry_new_with_max_length(len);
  gtk_widget_set_usize(intentries[num],len*8+5,0);
  sprintf(buf,"%d",*(intvals[num]));
  gtk_entry_set_text(GTK_ENTRY(intentries[num]),buf);
  gtk_signal_connect(GTK_OBJECT(intentries[num]),"changed",
  					 GTK_SIGNAL_FUNC(ChangeIntVal),(gpointer)num);

  gtk_box_pack_start(GTK_BOX(hbox),intentries[num],FALSE,FALSE,0);
  gtk_widget_show(intentries[num]);

  gtk_box_pack_start(GTK_BOX(box),hbox,FALSE,FALSE,0);
  gtk_widget_show(hbox);
}

void ChangeBoolVal(GtkWidget *widget,gpointer data)
{
  int *val;

  val=(int *)data;

  *val=1-*val;

  changed=1;
}

void MakeCheckButton(GtkWidget *box,char *name,int num)
{
  checkbuttons[num]=gtk_check_button_new_with_label(name);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(checkbuttons[num]),
												*(checkvals[num]));
  gtk_signal_connect(GTK_OBJECT(checkbuttons[num]),"clicked",
					 GTK_SIGNAL_FUNC(ChangeBoolVal),(gpointer)checkvals[num]);
  gtk_box_pack_start(GTK_BOX(box),checkbuttons[num],FALSE,FALSE,0);
  gtk_widget_show(checkbuttons[num]);
}

void MakeGenPage(void)
{
  GtkWidget *vbox;

  genpage=MakeNewPage("General");

  vbox=gtk_vbox_new(TRUE,0);

  gtk_container_border_width(GTK_CONTAINER(vbox),5);

  MakeCheckButton(vbox,"Disable splashscreen",0);
  MakeCheckButton(vbox,"Enable shamless plug",1);
  MakeCheckButton(vbox,"Do VGA passthrough",5);
  MakeCheckButton(vbox,"Disable shutdown",6);

  gtk_container_add(GTK_CONTAINER(genpage),vbox);
  gtk_widget_show(vbox);
}

void GammaSlide(GtkWidget *widget,gpointer data)
{
  int num;
  char buf[5];

  num=(int)data;

  *(gammavals[num])=GTK_ADJUSTMENT(widget)->value;

  sprintf(buf,"%3.3f",*(gammavals[num]));
  gtk_signal_handler_block_by_data(GTK_OBJECT(gammaentries[num]),
											 (gpointer)num);
  gtk_entry_set_text(GTK_ENTRY(gammaentries[num]),buf);
  gtk_signal_handler_unblock_by_data(GTK_OBJECT(gammaentries[num]),
											   (gpointer)num);

  changed=1;
}

void GammaEnter(GtkWidget *widget,gpointer data)
{
  int num;
  float val;

  num=(int)data;

  val=atof(gtk_entry_get_text(GTK_ENTRY(widget)));

  if(val<0.0) val=0.0;
  if(val>1.95) val=1.95;

  *(gammavals[num])=val;

  gtk_signal_handler_block_by_data(GTK_OBJECT(gammaadjs[num]),
								   (gpointer)num);
  gtk_adjustment_set_value(GTK_ADJUSTMENT(gammaadjs[num]),val);
  gtk_signal_handler_unblock_by_data(GTK_OBJECT(gammaadjs[num]),
									 (gpointer)num);

  changed=1;
}

void MakeGammaWidget(GtkWidget *box,char *name,int num)
{
  GtkWidget *gammabox;
  GtkWidget *scale;
  GtkWidget *label;
  char buf[5];

  gammabox=gtk_hbox_new(TRUE,5);

  label=gtk_label_new(name);
  gtk_box_pack_start(GTK_BOX(gammabox),label,FALSE,FALSE,0);
  gtk_widget_show(label);

  gammaadjs[num]=gtk_adjustment_new(*(gammavals[num]),0.0,1.95,.01,0.05,0.0);

  scale=gtk_hscale_new(GTK_ADJUSTMENT(gammaadjs[num]));
  gtk_scale_set_digits(GTK_SCALE(scale),3);
  gtk_widget_set_usize(GTK_WIDGET(scale),100,0);
  gtk_box_pack_start(GTK_BOX(gammabox),scale,FALSE,FALSE,0);
  gtk_scale_set_draw_value(GTK_SCALE(scale),FALSE);
  gtk_signal_connect(GTK_OBJECT(gammaadjs[num]),"value_changed",
					 GTK_SIGNAL_FUNC(GammaSlide),(gpointer)num);
  gtk_widget_show(scale);

  gammaentries[num]=gtk_entry_new_with_max_length(4);
  gtk_widget_set_usize(gammaentries[num],35,0);
  sprintf(buf,"%3.3f",*(gammavals[num]));
  gtk_entry_set_text(GTK_ENTRY(gammaentries[num]),buf);
  gtk_signal_connect(GTK_OBJECT(gammaentries[num]),"changed",
  					 GTK_SIGNAL_FUNC(GammaEnter),(gpointer)num);

  gtk_box_pack_start(GTK_BOX(gammabox),gammaentries[num],FALSE,FALSE,0);
  gtk_widget_show(gammaentries[num]);

  gtk_box_pack_start(GTK_BOX(box),gammabox,FALSE,FALSE,0);
  gtk_widget_show(gammabox);
}

void MakeColPage(void)
{
  GtkWidget *hbox,*vbox;

  colpage=MakeNewPage("Color");

  hbox=gtk_hbox_new(FALSE,5);
  gtk_container_border_width(GTK_CONTAINER(hbox),5);

  vbox=gtk_vbox_new(TRUE,5);
  gtk_container_border_width(GTK_CONTAINER(vbox),5);

  MakeGammaWidget(vbox,"Gamma",0);
  MakeGammaWidget(vbox,"Red gamma",1);
  MakeGammaWidget(vbox,"Green gamma",2);
  MakeGammaWidget(vbox,"Blue gamma",3);

  gtk_box_pack_start(GTK_BOX(hbox),vbox,FALSE,FALSE,0);
  gtk_widget_show(vbox);

  gtk_container_add(GTK_CONTAINER(colpage),hbox);
  gtk_widget_show(hbox);
}

void MakeAboutPage(void)
{
  GtkWidget *vbox,*vbox2;
  GtkWidget *label;
  GdkPixmap *logo;
  GdkBitmap *mask;
  GtkStyle *style;
  GtkWidget *pixmapwid;
  GtkWidget *ebox;
  char buf[256];

  style=gtk_widget_get_style(window);

  aboutpage=MakeNewPage("About");

  ebox=gtk_event_box_new();
  gtk_widget_set_name(ebox,"blackback");

  vbox=gtk_vbox_new(TRUE,5);
  gtk_container_border_width(GTK_CONTAINER(vbox),5);

  sprintf(buf,"%s/gc.xpm",AUXDIR);
  logo=gdk_pixmap_create_from_xpm(window->window,&mask,
								  &style->bg[GTK_STATE_NORMAL],
								  buf);

  pixmapwid=gtk_pixmap_new(logo,mask);
  gtk_box_pack_start(GTK_BOX(vbox),pixmapwid,FALSE,FALSE,0);
  gtk_widget_show(pixmapwid);

  vbox2=gtk_vbox_new(TRUE,0);

  label=gtk_label_new("Version 0.1");
  gtk_widget_set_name(label,"blackback");
  gtk_box_pack_start(GTK_BOX(vbox2),label,FALSE,FALSE,0);
  gtk_widget_show(label);

  label=gtk_label_new("Written by Mike Oliphant");
  gtk_widget_set_name(label,"blackback");
  gtk_box_pack_start(GTK_BOX(vbox2),label,FALSE,FALSE,0);
  gtk_widget_show(label);

  gtk_container_add(GTK_CONTAINER(vbox),vbox2);
  gtk_widget_show(vbox2);

  gtk_container_add(GTK_CONTAINER(ebox),vbox);
  gtk_widget_show(vbox);

  gtk_container_add(GTK_CONTAINER(aboutpage),ebox);
  gtk_widget_show(ebox);
}

void ReadBoolSet(char *name,int *val)
{
  char buf[80];
  char *env;

  if(*name=='_') {
	if(voodoo2) sprintf(buf,"SSTV2%s",name);
	else sprintf(buf,"SST%s",name);

	env=getenv(buf);
  }
  else env=getenv(name);

  *val=(env)?1:0;
}

void ReadIntSet(char *name,int *val)
{
  char buf[80];
  char *env;

  if(*name=='_') {
	if(voodoo2) sprintf(buf,"SSTV2%s",name);
	else sprintf(buf,"SST%s",name);

	env=getenv(buf);
  }
  else env=getenv(name);

  if(env) *val=atoi(env);
}

void ReadFloatSet(char *name,float *val)
{
  char buf[80];
  char *env;

  if(*name=='_') {
	if(voodoo2) sprintf(buf,"SSTV2%s",name);
	else sprintf(buf,"SST%s",name);

	env=getenv(buf);
  }
  else env=getenv(name);

  if(env) *val=atof(env);
}

void ReadSettings(void)
{
  ReadIntSet("FX_GLIDE_NO_SPLASH",&nosplash);
  ReadBoolSet("FX_GLIDE_SHAMELESS_PLUG",&shameplug);

  ReadIntSet("_VGA_PASS",&vgapass);

  ReadBoolSet("_NOSHUTDOWN",&noshutdown);

  ReadFloatSet("_GAMMA",&maingamma);

  ReadFloatSet("_RGAMMA",&rgamma);
  ReadFloatSet("_GGAMMA",&ggamma);
  ReadFloatSet("_BGAMMA",&bgamma);

  if(voodoo2)
	ReadIntSet("SSTV2_FASTMEM_RAS_READS",&fastmem);
  else ReadIntSet("SST_FASTMEM",&fastmem);

  if(voodoo2)
	ReadIntSet("SSTV2_FASTPCIRD",&fastpcird);
  else ReadIntSet("SST_PCIRD",&fastpcird);

  ReadIntSet("_SWAP_EN_WAIT_ON_VSYNC",&vsync);

  ReadIntSet("FX_GLIDE_SWAPINTERVAL",&swapint);
  ReadIntSet("_GRXCLK",&grxclk);
  ReadIntSet("_SCREENREFRESH",&refresh);
}

void SaveBoolSet(char *name,int val,FILE *fp)
{
  char buf[80];

  if(*name=='_') {
	if(voodoo2) sprintf(buf,"SSTV2%s",name);
	else sprintf(buf,"SST%s",name);
	
	if(val) {
	  fprintf(fp,"%s=\n",buf);
	  fprintf(fp,"export %s\n\n",buf);
	}
	else fprintf(fp,"unset %s\n\n",buf);
  }
  else {
	if(val) {
	  fprintf(fp,"%s=\n",name);
	  fprintf(fp,"export %s\n\n",name);
	}
	else fprintf(fp,"unset %s\n\n",name);
	
  }
}

void SaveIntSet(char *name,int val,FILE *fp)
{
  char buf[80];

  if(*name=='_') {
	if(voodoo2) sprintf(buf,"SSTV2%s",name);
	else sprintf(buf,"SST%s",name);

	fprintf(fp,"%s=%d\n",buf,val);
	fprintf(fp,"export %s\n\n",buf);
  }
  else {
	fprintf(fp,"%s=%d\n",name,val);
	fprintf(fp,"export %s\n\n",name);
  }
}

void SaveFloatSet(char *name,float val,FILE *fp)
{
  char buf[80];

  if(*name=='_') {
	if(voodoo2) sprintf(buf,"SSTV2%s",name);
	else sprintf(buf,"SST%s",name);

	fprintf(fp,"%s=%3.2f\n",buf,val);
	fprintf(fp,"export %s\n\n",buf);
  }
  else {
	fprintf(fp,"setenv %s %3.2f\n",name,val);
	fprintf(fp,"export %s\n\n",name);
  }
}

void SaveSettings(FILE *fp)
{
  SaveIntSet("FX_GLIDE_NO_SPLASH",nosplash,fp);
  SaveBoolSet("FX_GLIDE_SHAMELESS_PLUG",shameplug,fp);

  SaveIntSet("_VGA_PASS",vgapass,fp);
  SaveBoolSet("_NOSHUTDOWN",noshutdown,fp);

  SaveFloatSet("_GAMMA",maingamma,fp);
  SaveFloatSet("_RGAMMA",rgamma,fp);
  SaveFloatSet("_GGAMMA",ggamma,fp);
  SaveFloatSet("_BGAMMA",bgamma,fp);

  if(voodoo2)
	SaveIntSet("SSTV2_FASTMEM_RAS_READS",fastmem,fp);
  else SaveIntSet("SST_FASTMEM",fastmem,fp);

  if(voodoo2)
	SaveIntSet("SSTV2_FASTPCIRD",fastpcird,fp);
  else SaveIntSet("SST_PCIRD",fastpcird,fp);

  SaveIntSet("_SWAP_EN_WAIT_ON_VSYNC",vsync,fp);

  SaveIntSet("FX_GLIDE_SWAPINTERVAL",swapint,fp);
  SaveIntSet("_GRXCLK",grxclk,fp);
  SaveIntSet("_SCREENREFRESH",refresh,fp);
}

void DoSave(GtkWidget *widget,gpointer data)
{
  FILE *fp;

  fp=fopen("/etc/profile.d/glidecontrol.sh","w");

  if(fp) {
	SaveSettings(fp);
	fclose(fp);
  }
  else
	printf("Error: could not write to /etc/profile.d/glidecontrol.sh\n");

  changed=0;
}

int main(int argc, char *argv[])
{
  GtkWidget *button;
  GtkWidget *vbox,*hbox;
  char *env;
  char buf[256];

  env=getenv("SHELL");
  if(env&&strcasecmp(env,"/bin/bash"))
	printf("** WARNING! -- GlideControl only works with bash\n");

  if(grSstQueryBoards(&hwconfig)!=FXTRUE) {
	printf("Error: No 3Dfx hardware found\n");

	return 0;
  }

  grGlideInit();
  grSstQueryHardware(&hwconfig);

  grGlideGetVersion(version);

  grGlideShutdown();

  ReadSettings();
       
  gtk_init(&argc, &argv);

  sprintf(buf,"%s/GlideControl.rc",AUXDIR);
  gtk_rc_parse(buf);

  window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window),"GlideControl");

  gtk_signal_connect(GTK_OBJECT(window),"delete_event",
					 GTK_SIGNAL_FUNC(ShutDown),NULL);


  vbox=gtk_vbox_new(FALSE,5);
  gtk_container_border_width(GTK_CONTAINER(vbox),5);

  notebook=gtk_notebook_new();

  gtk_box_pack_start(GTK_BOX(vbox),notebook,FALSE,FALSE,0);

  MakeInfoPage();
  MakeGenPage();
  MakeSpeedPage();
  MakeColPage();

  gtk_widget_show(notebook);

  hbox=gtk_hbox_new(TRUE,5);

  button=gtk_button_new_with_label("Save current glide settings");
  gtk_signal_connect(GTK_OBJECT(button),"clicked",
					 GTK_SIGNAL_FUNC(DoSave),NULL);

  gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
  gtk_widget_show(button);

  button=gtk_button_new_with_label("Exit GlideControl");
  gtk_signal_connect(GTK_OBJECT(button),"clicked",
					 GTK_SIGNAL_FUNC(ShutDown),NULL);

  gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
  gtk_widget_show(button);

  gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
  gtk_widget_show(hbox);

  gtk_container_add(GTK_CONTAINER(window),vbox);
  gtk_widget_show(vbox);

  gtk_widget_show(window);

  /* Put this here so that the pixmap routines don't complain */

  MakeAboutPage();

  gtk_main();
  
  return 0;
}
